import { ITypeBuilder } from './ITypeBuilder';
import { Token, IContainer, Type } from '@ts-ioc/core';
import { TypeConfigure } from './TypeConfigure';
/**
 * token bootstrap builder. build class with metadata and config.
 *
 * @export
 * @class BootBuilder
 * @implements {implements ITypeBuilder<T>}
 * @template T
 */
export declare class TypeBuilder<T> implements ITypeBuilder<T> {
    /**
     * ioc container.
     *
     * @type {IContainer}
     * @memberof BootBuilder
     */
    container: IContainer;
    constructor();
    build(token: Token<T>, config?: TypeConfigure<T>, data?: any): Promise<T>;
    buildByConfig(config: Token<T> | TypeConfigure<T>, data?: any): Promise<any>;
    createInstance(token: Token<T>, config: TypeConfigure<T>, data?: any): Promise<T>;
    getBuilder(config: TypeConfigure<T>): ITypeBuilder<T>;
    /**
     * bundle instance via config.
     *
     * @param {T} instance
     * @param {TypeConfigure} config
     * @param {IContainer} [container]
     * @returns {Promise<T>}
     * @memberof BootBuilder
     */
    buildStrategy(instance: T, config: TypeConfigure<T>): Promise<T>;
    getBootstrapToken(config: TypeConfigure<T>): Token<T>;
    protected getTokenMetaConfig(token: Token<T>, config?: TypeConfigure<T>): TypeConfigure<T>;
    getDecorator(): string;
    protected getMetaConfig(token: Type<any>): TypeConfigure<T>;
    protected resolveToken(token: Token<T>, data?: any): T;
    static classAnnations: any;
}
