"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var ITypeBuilder_1 = require("./ITypeBuilder");
var core_1 = require("@ts-ioc/core");
var decorators_1 = require("./decorators");
/**
 * token bootstrap builder. build class with metadata and config.
 *
 * @export
 * @class BootBuilder
 * @implements {implements ITypeBuilder<T>}
 * @template T
 */
var TypeBuilder = /** @class */ (function () {
    function TypeBuilder() {
    }
    TypeBuilder_1 = TypeBuilder;
    TypeBuilder.prototype.build = function (token, config, data) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var builder, instance;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!config) {
                            config = this.getTokenMetaConfig(token);
                        }
                        builder = this.getBuilder(config);
                        if (!(builder !== this)) return [3 /*break*/, 1];
                        return [2 /*return*/, builder.build(token, config, data)];
                    case 1: return [4 /*yield*/, this.createInstance(token, config, data)];
                    case 2:
                        instance = _a.sent();
                        return [4 /*yield*/, this.buildStrategy(instance, config)];
                    case 3:
                        instance = _a.sent();
                        return [2 /*return*/, instance];
                }
            });
        });
    };
    TypeBuilder.prototype.buildByConfig = function (config, data) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var token;
            return tslib_1.__generator(this, function (_a) {
                if (core_1.isToken(config)) {
                    token = config;
                    return [2 /*return*/, this.build(token, this.getTokenMetaConfig(token), data)];
                }
                else {
                    token = this.getBootstrapToken(config);
                    return [2 /*return*/, this.build(token, this.getTokenMetaConfig(token, config), data)];
                }
                return [2 /*return*/];
            });
        });
    };
    TypeBuilder.prototype.createInstance = function (token, config, data) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var instance;
            return tslib_1.__generator(this, function (_a) {
                if (!token) {
                    throw new Error('cant not find bootstrap token.');
                }
                if (!this.container.has(token)) {
                    if (core_1.isClass(token)) {
                        console.log('boot builder', token);
                        this.container.register(token);
                    }
                    else {
                        throw new Error("cant not find token " + token.toString() + " in container.");
                    }
                }
                instance = this.resolveToken(token, data);
                return [2 /*return*/, instance];
            });
        });
    };
    TypeBuilder.prototype.getBuilder = function (config) {
        if (config && config.typeBuilder) {
            if (core_1.isClass(config.typeBuilder)) {
                if (!this.container.has(config.typeBuilder)) {
                    this.container.register(config.typeBuilder);
                }
            }
            if (core_1.isToken(config.typeBuilder)) {
                return this.container.resolve(config.typeBuilder);
            }
            else if (config.typeBuilder instanceof TypeBuilder_1) {
                return config.typeBuilder;
            }
        }
        return this;
    };
    /**
     * bundle instance via config.
     *
     * @param {T} instance
     * @param {TypeConfigure} config
     * @param {IContainer} [container]
     * @returns {Promise<T>}
     * @memberof BootBuilder
     */
    TypeBuilder.prototype.buildStrategy = function (instance, config) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                return [2 /*return*/, instance];
            });
        });
    };
    TypeBuilder.prototype.getBootstrapToken = function (config) {
        return config.bootstrap;
    };
    TypeBuilder.prototype.getTokenMetaConfig = function (token, config) {
        var cfg;
        if (core_1.isClass(token)) {
            cfg = this.getMetaConfig(token);
        }
        else if (core_1.isToken(token)) {
            var tokenType = this.container ? this.container.getTokenImpl(token) : token;
            if (core_1.isClass(tokenType)) {
                cfg = this.getMetaConfig(tokenType);
            }
        }
        if (cfg) {
            return core_1.lang.assign({}, cfg, config || {});
        }
        else {
            return config || {};
        }
    };
    TypeBuilder.prototype.getDecorator = function () {
        return decorators_1.Build.toString();
    };
    TypeBuilder.prototype.getMetaConfig = function (token) {
        var decorator = this.getDecorator();
        if (core_1.hasOwnClassMetadata(decorator, token)) {
            var metas = core_1.getTypeMetadata(decorator, token);
            if (metas && metas.length) {
                return metas[0];
            }
        }
        return null;
    };
    TypeBuilder.prototype.resolveToken = function (token, data) {
        return this.container.resolve(token, data);
    };
    var TypeBuilder_1;
    TypeBuilder.classAnnations = { "name": "TypeBuilder", "params": { "constructor": [], "build": ["token", "config", "data"], "buildByConfig": ["config", "data"], "createInstance": ["token", "config", "data"], "getBuilder": ["config"], "buildStrategy": ["instance", "config"], "getBootstrapToken": ["config"], "getTokenMetaConfig": ["token", "config"], "getDecorator": [], "getMetaConfig": ["token"], "resolveToken": ["token", "data"] } };
    tslib_1.__decorate([
        core_1.Inject(core_1.ContainerToken),
        tslib_1.__metadata("design:type", Object)
    ], TypeBuilder.prototype, "container", void 0);
    TypeBuilder = TypeBuilder_1 = tslib_1.__decorate([
        core_1.Singleton(ITypeBuilder_1.TypeBuilderToken),
        tslib_1.__metadata("design:paramtypes", [])
    ], TypeBuilder);
    return TypeBuilder;
}());
exports.TypeBuilder = TypeBuilder;

//# sourceMappingURL=sourcemaps/TypeBuilder.js.map
