import { ClassMetadata, ITypeDecorator, Token, MetadataAdapter, MetadataExtends } from '@ts-ioc/core';
import { ITypeBuilder } from '../ITypeBuilder';
import { TypeConfigure } from '../TypeConfigure';
export interface BuildMetadata extends ClassMetadata, TypeConfigure<any> {
}
/**
 * Build decorator, use to define class build way via config.
 *
 * @export
 * @interface IBuildDecorator
 * @extends {ITypeDecorator<T>}
 * @template T
 */
export interface IBuildDecorator<T extends BuildMetadata> extends ITypeDecorator<T> {
    /**
     * Build decorator, use to define class as DI Module.
     *
     * @Build
     *
     * @param {T} [metadata] bootstrap metadate config.
     */
    (metadata: T): ClassDecorator;
}
/**
 * create type builder decorator
 *
 * @export
 * @template T
 * @param {string} name
 * @param {string} [decorType]
 * @param {(Token<ITypeBuilder<any>> | ITypeBuilder<any>)} [builder]
 * @param {MetadataAdapter} [adapter]
 * @param {MetadataExtends<T>} [metadataExtends]
 * @returns {IBuildDecorator<T>}
 */
export declare function createBuildDecorator<T extends BuildMetadata>(name: string, builder?: Token<ITypeBuilder<any>> | ITypeBuilder<any>, adapter?: MetadataAdapter, metadataExtends?: MetadataExtends<T>): IBuildDecorator<T>;
/**
 * Build decorator, use to define class build way via config.
 *
 * @Build
 */
export declare const Build: IBuildDecorator<BuildMetadata>;
