"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@ts-ioc/core");
/**
 * create type builder decorator
 *
 * @export
 * @template T
 * @param {string} name
 * @param {string} [decorType]
 * @param {(Token<ITypeBuilder<any>> | ITypeBuilder<any>)} [builder]
 * @param {MetadataAdapter} [adapter]
 * @param {MetadataExtends<T>} [metadataExtends]
 * @returns {IBuildDecorator<T>}
 */
function createBuildDecorator(name, builder, adapter, metadataExtends) {
    return core_1.createClassDecorator(name, function (args) {
        if (adapter) {
            adapter(args);
        }
    }, function (metadata) {
        if (metadataExtends) {
            metadata = metadataExtends(metadata);
        }
        if (builder && !metadata.typeBuilder) {
            metadata.typeBuilder = builder;
        }
        return metadata;
    });
}
exports.createBuildDecorator = createBuildDecorator;
/**
 * Build decorator, use to define class build way via config.
 *
 * @Build
 */
exports.Build = createBuildDecorator('Build');

//# sourceMappingURL=../sourcemaps/decorators/Build.js.map
